@@ -8,7 +8,7 @@
 #' @param map_token character, authentification token for services that require registration, which are \code{"osm_thunderforest"} and \code{"mapbox"}. Register at \url{https://www.thunderforest.com/} and/or \url{https://www.mapbox.com/} to get tokens. Ignored for all other map services.
 #' @param map_res numeric, resolution of base map in range from 0 to 1.
 #' @param map_dir character, cache directory where downloaded basemap tiles will be stored. By default, a temporary directory is used, which is destroyed when the session is terminated.
-#' @param class character, output class, either \code{"stars"}, \code{"raster"}, \code{"mapview"}, \code{"plot"}, \code{"ggplot"}, \code{"gglayer"}, \code{"magick"}, \code{"png"} or \code{"geotif"}..
+#' @param class character, output class, either either \code{plot} (default), \code{magick}, \code{png}, \code{geotif} or if suggested packages are installed, \code{terra}, \code{raster}, \code{stars}, \code{ggplot}, \code{gglayer} or \code{mapview}.
 #' @param force logical, whether to force download over cached files or not. Default is \code{FALSE}.
 #' @param ... additional arguments, including
 #' \itemize{
@@ -80,8 +80,7 @@
 #'   scale_fill_identity()
 #'  }
 #' @importFrom sf st_bbox 
-#' @importFrom raster nlayers brick raster
-#' @importFrom stars read_stars
+#' @importFrom terra rast plotRGB plot as.array nlyr
 #' @importFrom graphics plot
 #' @importFrom magick image_read
 #' @importFrom grDevices topo.colors col2rgb
@@ -89,13 +88,16 @@
 #' @export
 #' @name basemap
 basemap <- function(ext = NULL, map_service = NULL, map_type = NULL, map_res = NULL, map_token = NULL,
-                    map_dir = NULL, class = "stars", force = FALSE, ..., verbose = TRUE){
+                    map_dir = NULL, class = "plot", force = FALSE, ..., verbose = TRUE){
   
   ## checks
   if(inherits(verbose, "logical")) options(basemaps.verbose = verbose)
   if(is.null(ext)) ext <- getOption("basemaps.defaults")$ext
   if(is.null(ext)) out("Argument 'ext' is not defined and there is no default defined using set_defaults().", type = 3)
-  if(inherits(ext, "sf")) ext <- st_bbox(ext)
+  if(!inherits(ext, "list")) ext <- list(ext)
+  ext <- lapply(ext, function(x){
+    if(inherits(x, "sf")) st_bbox(x) else x
+  })
   if(is.null(map_service)) map_service <- getOption("basemaps.defaults")$map_service
   if(is.null(map_type)) map_type <- getOption("basemaps.defaults")$map_type
   if(is.null(map_res)) map_res <- getOption("basemaps.defaults")$map_res
@@ -119,10 +121,12 @@ basemap <- function(ext = NULL, map_service = NULL, map_type = NULL, map_res = N
   
   ## transform ext if needed
   crs_webmerc <- st_crs(3857)
-  if(st_crs(ext) != crs_webmerc){
+  if(any(sapply(ext, function(x) st_crs(x) != crs_webmerc, USE.NAMES = F))){
     out(paste0("Transforming 'ext' to Web Mercator (EPSG: 3857), since 'ext' has a different CRS. The CRS of the returned basemap will be Web Mercator, which is the default CRS used by the supported tile services."), type = 2)
   }
-  ext <- st_bbox(st_transform(st_as_sfc(st_bbox(ext)), crs = crs_webmerc)) #bbox as web mercator, always
+  ext <- lapply(ext, function(x){
+    st_bbox(st_transform(st_as_sfc(st_bbox(x)), crs = crs_webmerc)) #bbox as web mercator, always
+  })
   
   ## get map
   out(paste0("Loading basemap '", map_type, "' from map service '", map_service, "'..."))
@@ -131,48 +135,51 @@ basemap <- function(ext = NULL, map_service = NULL, map_type = NULL, map_res = N
   # return file if needed
   if("geotif" %in% class) return(map_file)
   
-  ## stars-based:
-  if(any(c("stars", "plot", "png", "magick") %in% class)){
-    map <- read_stars(map_file)
+  # return terra
+  if(any(c("terra", "plot", "png", "magick", "ggplot", "gglayer") %in% class)){
+    map <- terra::rast(map_file)
     
-    if("stars" %in% class) return(map)
-    if("plot" == class){
-      dim_map <- dim(map)
-      if(length(dim(map)) == 2) dim_map["band"] <- 1
+    if("terra" %in% class) return(map)
+    
+    if("plot" %in% class){
+      dim_map <- dim(map) 
+      
       if(dim_map[3] == 3){
-        # avoid failure if only single value is present
-        if(length(unique(range(map[[1]][]))) == 1){
-          plot(map, rgb = 1:3, main = NULL, downsample = 0, maxColorValue=max(map[[1]][])+1) 
-        } else{
-          plot(map, rgb = 1:3, main = NULL, downsample = 0) 
-        }
+        terra::plotRGB(map, r = 1, g = 2, b = 3, maxcell = ncell(map))
       } else{
-        plot(map, col = col, 
-             breaks = seq(min(map[[1]]), max(map[[1]]), length.out = length(col)+1),
-             main = NULL, downsample = 0)
+        terra::plot(
+          map[[1]],
+          col = col, type = "continous",
+          breaks = seq(min(map[[1]][]), max(map[[1]][]), length.out = length(col)+1)
+        )
       }
     }
+    
     if(any("png" == class, "magick" == class)){
       if(!any(grepl("png", rownames(installed.packages())))){
-        out("Package 'png' is not installed. Please install 'png' using install.packages('png').")
+        out(paste0("Package 'png' is not installed, but needed for class='", class, "'. Please install 'png' using install.packages('png')."), type = 3)
       } else{
-        file <- paste0(map_dir, "/", map_service, "_", map_type, "_", gsub(":", "-", gsub(" ", "_", Sys.time())), ".png")
-        map_arr <- map[[1]]
         
-        if(!is.na(dim(map_arr)[3])){
+        file <- paste0(map_dir, "/", map_service, "_", map_type, "_", gsub(":", "-", gsub(" ", "_", Sys.time())), ".png")
+        map_arr <- terra::as.array(map)
+        
+        if(dim(map_arr)[3] == 3){
           #for(i in 1:dim(map_arr)[3]) map_arr[,,i] <- t(map_arr[,,i])
-          map_arr <- aperm(map_arr, c(2, 1, 3))
+          #map_arr <- aperm(map_arr, c(2, 1, 3)) ### ONLY WITH STARS
           map_arr <- sweep(map_arr, MARGIN = 3, STATS = max(map_arr), FUN = "/")
           png::writePNG(map_arr, target = file, dpi = dpi)
         } else{
+          map_arr <- map_arr[,,1]
           # convert to range 0 to 1
-          map_arr <- sweep(t(map_arr), MARGIN = 1, STATS = max(map_arr), FUN = "/")
+          #map_arr <- sweep(map_arr, MARGIN = 1, STATS = max(map_arr), FUN = "/")
+          map_arr <- ((map_arr - min(map_arr))/(max(map_arr) - min(map_arr)))
           # map col to value range
-          map_arr_col <- col[findInterval(map_arr, seq(0, 1, length.out = length(col)))]#, dim(map_arr))
+          map_arr_col <- col[findInterval(map_arr, seq(0, 1, length.out = length(col)))]
           # convert hex to rgb
           map_arr_rgb <- col2rgb(map_arr_col)
           # switch dimensions to fit writeRGB
           map_arr_rgb <- aperm(array(map_arr_rgb, c(3, dim(map_arr))), c(2,3,1))
+          #map_arr_rgb <- array(map_arr_rgb, c(dim(map_arr), 3))
           # go back to 0 to 1 again
           map_arr_rgb <- sweep(map_arr_rgb, MARGIN = 3, STATS = max(map_arr_rgb), FUN = "/")
           png::writePNG(map_arr_rgb, target = file, dpi = dpi)
@@ -186,65 +193,63 @@ basemap <- function(ext = NULL, map_service = NULL, map_type = NULL, map_res = N
         }
       }
     }
+    
+    if("ggplot" %in% class){
+      if(!any(grepl("ggplot", rownames(installed.packages())))){
+        out(paste0("Package 'ggplot2' is not installed, but needed for class='", class, "'. Please install 'ggplot2' using install.packages('ggplot2')."), type = 3)
+      } else{
+        if(terra::nlyr(map) == 3){
+          return(gg_raster(r = map, r_type = "RGB", ...))
+        } else{
+          return(gg_raster(r = map, r_type = "gradient", ...))
+        }
+      }
+    }
+    
+    if("gglayer" %in% class){
+      if(!any(grepl("ggplot", rownames(installed.packages())))){
+        out(paste0("Package 'ggplot2' is not installed, but needed for class='", class, "'. Please install 'ggplot2' using install.packages('ggplot2')."), type = 3)
+      } else{
+        if(terra::nlyr(map) == 3) return(gg_raster(r = map, r_type = "RGB", gglayer = T, ...)) else return(gg_raster(r = map, r_type = "gradient", gglayer = T, ...))
+      }
+    }
+  }
+  
+  # stars
+  if("stars" %in% class){
+    if(!any(grepl("stars", rownames(installed.packages())))){
+      out(paste0("Package 'stars' is not installed, but needed for class='", class, "'. Please install 'stars' using install.packages('stars')."), type = 3)
+    } else{
+      map <- stars::read_stars(map_file)
+      return(map)
+    }
   }
   
   # raster-based
   if(any(c("raster", "mapview", "ggplot", "gglayer") %in% class)){
-    map <- quiet(brick(map_file))
-    if("raster" %in% class){
-      if(nlayers(map) == 1) map <- raster(map)
-      return(map)
-    }
-    
-    if("mapview" %in% class){
-      if(!any(grepl("mapview", rownames(installed.packages())))){
-        out("Package 'mapview' is not installed. Please install 'mapview' using install.packages('mapview').")
-      } else{
-        quiet(if(nlayers(map) == 3){
-          return(mapview::viewRGB(map, 1, 2, 3, layer.name = "Basemap", maxpixels = ncell(map), quantiles = NULL))
-        } else return(mapview::mapview(map)))
+    if(!any(grepl("raster", rownames(installed.packages())))){
+      out(paste0("Package 'raster' is not installed, but needed for class='", class, "'. Please install 'raster' using install.packages('raster')."), type = 3)
+    } else{
+      
+      map <- quiet(raster::brick(map_file))
+      if("raster" %in% class){
+        if(raster::nlayers(map) == 1) map <- raster::raster(map)
+        return(map)
       }
-    }
-    
-    if("ggplot" %in% class){
-      if(!any(grepl("ggplot", rownames(installed.packages())))){
-        out("Package 'ggplot2' is not installed. Please install 'ggplot2' using install.packages('ggplot2').")
-      } else{
-        if(nlayers(map) == 3){
-          return(gg.bmap(r = map, r_type = "RGB", ...))
+      
+      if("mapview" %in% class){
+        if(!any(grepl("mapview", rownames(installed.packages())))){
+          out(paste0("Package 'mapview' is not installed, but needed for class='", class, "'. Please install 'mapview' using install.packages('mapview')."), type = 3)
         } else{
-          return(gg.bmap(r = map, r_type = "gradient", ...))
+          quiet(if(raster::nlayers(map) == 3){
+            return(mapview::viewRGB(map, 1, 2, 3, layer.name = "Basemap", maxpixels = raster::ncell(map), quantiles = NULL))
+          } else return(mapview::mapview(map)))
         }
       }
     }
-  
-    if("gglayer" %in% class){
-      if(!any(grepl("ggplot", rownames(installed.packages())))){
-        out("Package 'ggplot2' is not installed. Please install 'ggplot2' using install.packages('ggplot2').")
-      } else{
-        if(nlayers(map) == 3) return(gg.bmap(r = map, r_type = "RGB", gglayer = T, ...)) else return(gg.bmap(r = map, r_type = "gradient", gglayer = T, ...))
-      }
-    }
   }
 }
 
-#' @rdname basemap
-#' @export
-basemap_raster <- function(ext = NULL, map_service = NULL, map_type = NULL, map_res = NULL, map_token = NULL, map_dir = NULL, force = NULL, ..., verbose = TRUE){
-  basemap(ext, map_service, map_type, map_res, map_token, map_dir, class = "raster", force, ..., verbose = verbose)
-}
-
-#' @rdname basemap
-#' @export
-basemap_stars <- function(ext = NULL, map_service = NULL, map_type = NULL, map_res = NULL, map_token = NULL, map_dir = NULL, force = NULL, ..., verbose = TRUE){
-  basemap(ext, map_service, map_type, map_res, map_token, map_dir, class = "stars", force, ..., verbose = verbose)
-}
-
-#' @rdname basemap
-#' @export
-basemap_mapview <- function(ext = NULL, map_service = NULL, map_type = NULL, map_res = NULL, map_token = NULL, map_dir = NULL, force = NULL, ..., verbose = TRUE){
-  basemap(ext, map_service, map_type, map_res, map_token, map_dir, class = "mapview", force, ..., verbose = verbose)
-}
 
 #' @rdname basemap
 #' @export
@@ -252,18 +257,6 @@ basemap_plot <- function(ext = NULL, map_service = NULL, map_type = NULL, map_re
   basemap(ext, map_service, map_type, map_res, map_token, map_dir, class = "plot", force, ..., verbose = verbose)
 }
 
-#' @rdname basemap
-#' @export
-basemap_ggplot <- function(ext = NULL, map_service = NULL, map_type = NULL, map_res = NULL, map_token = NULL, map_dir = NULL, force = NULL, ..., verbose = TRUE){
-  basemap(ext, map_service, map_type, map_res, map_token, map_dir, class = "ggplot", force, ..., verbose = verbose)
-}
-
-#' @rdname basemap
-#' @export
-basemap_gglayer <- function(ext = NULL, map_service = NULL, map_type = NULL, map_res = NULL, map_token = NULL, map_dir = NULL, force = NULL, ..., verbose = TRUE){
-  basemap(ext, map_service, map_type, map_res, map_token, map_dir, class = "gglayer", force, ..., verbose = verbose)
-}
-
 #' @rdname basemap
 #' @export
 basemap_magick <- function(ext = NULL, map_service = NULL, map_type = NULL, map_res = NULL, map_token = NULL, map_dir = NULL, force = NULL, ..., verbose = TRUE){
@@ -280,4 +273,41 @@ basemap_png <- function(ext = NULL, map_service = NULL, map_type = NULL, map_res
 #' @export
 basemap_geotif <- function(ext = NULL, map_service = NULL, map_type = NULL, map_res = NULL, map_token = NULL, map_dir = NULL, force = NULL, ..., verbose = TRUE){
   basemap(ext, map_service, map_type, map_res, map_token, map_dir, class = "geotif", force, ..., verbose = verbose)
-}
\ No newline at end of file
+}
+
+#' @rdname basemap
+#' @export
+basemap_terra <- function(ext = NULL, map_service = NULL, map_type = NULL, map_res = NULL, map_token = NULL, map_dir = NULL, force = NULL, ..., verbose = TRUE){
+  basemap(ext, map_service, map_type, map_res, map_token, map_dir, class = "terra", force, ..., verbose = verbose)
+}
+
+#' @rdname basemap
+#' @export
+basemap_raster <- function(ext = NULL, map_service = NULL, map_type = NULL, map_res = NULL, map_token = NULL, map_dir = NULL, force = NULL, ..., verbose = TRUE){
+  basemap(ext, map_service, map_type, map_res, map_token, map_dir, class = "raster", force, ..., verbose = verbose)
+}
+
+#' @rdname basemap
+#' @export
+basemap_stars <- function(ext = NULL, map_service = NULL, map_type = NULL, map_res = NULL, map_token = NULL, map_dir = NULL, force = NULL, ..., verbose = TRUE){
+  basemap(ext, map_service, map_type, map_res, map_token, map_dir, class = "stars", force, ..., verbose = verbose)
+}
+
+
+#' @rdname basemap
+#' @export
+basemap_ggplot <- function(ext = NULL, map_service = NULL, map_type = NULL, map_res = NULL, map_token = NULL, map_dir = NULL, force = NULL, ..., verbose = TRUE){
+  basemap(ext, map_service, map_type, map_res, map_token, map_dir, class = "ggplot", force, ..., verbose = verbose)
+}
+
+#' @rdname basemap
+#' @export
+basemap_gglayer <- function(ext = NULL, map_service = NULL, map_type = NULL, map_res = NULL, map_token = NULL, map_dir = NULL, force = NULL, ..., verbose = TRUE){
+  basemap(ext, map_service, map_type, map_res, map_token, map_dir, class = "gglayer", force, ..., verbose = verbose)
+}
+
+#' @rdname basemap
+#' @export
+basemap_mapview <- function(ext = NULL, map_service = NULL, map_type = NULL, map_res = NULL, map_token = NULL, map_dir = NULL, force = NULL, ..., verbose = TRUE){
+  basemap(ext, map_service, map_type, map_res, map_token, map_dir, class = "mapview", force, ..., verbose = verbose)
+}
